// Copyright 2016 PDFium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Original code copyright 2014 Foxit Software Inc. http://www.foxitsoftware.com

#include "xfa/fxfa/parser/xfa_basic_data.h"

#include "fxjs/xfa/cjx_arc.h"
#include "fxjs/xfa/cjx_area.h"
#include "fxjs/xfa/cjx_assist.h"
#include "fxjs/xfa/cjx_barcode.h"
#include "fxjs/xfa/cjx_bind.h"
#include "fxjs/xfa/cjx_binditems.h"
#include "fxjs/xfa/cjx_bookend.h"
#include "fxjs/xfa/cjx_boolean.h"
#include "fxjs/xfa/cjx_border.h"
#include "fxjs/xfa/cjx_break.h"
#include "fxjs/xfa/cjx_breakafter.h"
#include "fxjs/xfa/cjx_breakbefore.h"
#include "fxjs/xfa/cjx_button.h"
#include "fxjs/xfa/cjx_calculate.h"
#include "fxjs/xfa/cjx_caption.h"
#include "fxjs/xfa/cjx_certificate.h"
#include "fxjs/xfa/cjx_certificates.h"
#include "fxjs/xfa/cjx_checkbutton.h"
#include "fxjs/xfa/cjx_choicelist.h"
#include "fxjs/xfa/cjx_color.h"
#include "fxjs/xfa/cjx_comb.h"
#include "fxjs/xfa/cjx_command.h"
#include "fxjs/xfa/cjx_connect.h"
#include "fxjs/xfa/cjx_connectstring.h"
#include "fxjs/xfa/cjx_contentarea.h"
#include "fxjs/xfa/cjx_corner.h"
#include "fxjs/xfa/cjx_datavalue.h"
#include "fxjs/xfa/cjx_datawindow.h"
#include "fxjs/xfa/cjx_date.h"
#include "fxjs/xfa/cjx_datetime.h"
#include "fxjs/xfa/cjx_datetimeedit.h"
#include "fxjs/xfa/cjx_decimal.h"
#include "fxjs/xfa/cjx_defaultui.h"
#include "fxjs/xfa/cjx_delete.h"
#include "fxjs/xfa/cjx_delta.h"
#include "fxjs/xfa/cjx_desc.h"
#include "fxjs/xfa/cjx_digestmethod.h"
#include "fxjs/xfa/cjx_digestmethods.h"
#include "fxjs/xfa/cjx_draw.h"
#include "fxjs/xfa/cjx_edge.h"
#include "fxjs/xfa/cjx_encoding.h"
#include "fxjs/xfa/cjx_encodings.h"
#include "fxjs/xfa/cjx_encrypt.h"
#include "fxjs/xfa/cjx_event.h"
#include "fxjs/xfa/cjx_eventpseudomodel.h"
#include "fxjs/xfa/cjx_exclgroup.h"
#include "fxjs/xfa/cjx_exdata.h"
#include "fxjs/xfa/cjx_execute.h"
#include "fxjs/xfa/cjx_exobject.h"
#include "fxjs/xfa/cjx_extras.h"
#include "fxjs/xfa/cjx_field.h"
#include "fxjs/xfa/cjx_fill.h"
#include "fxjs/xfa/cjx_filter.h"
#include "fxjs/xfa/cjx_float.h"
#include "fxjs/xfa/cjx_font.h"
#include "fxjs/xfa/cjx_format.h"
#include "fxjs/xfa/cjx_handler.h"
#include "fxjs/xfa/cjx_hostpseudomodel.h"
#include "fxjs/xfa/cjx_image.h"
#include "fxjs/xfa/cjx_imageedit.h"
#include "fxjs/xfa/cjx_insert.h"
#include "fxjs/xfa/cjx_instancemanager.h"
#include "fxjs/xfa/cjx_integer.h"
#include "fxjs/xfa/cjx_issuers.h"
#include "fxjs/xfa/cjx_items.h"
#include "fxjs/xfa/cjx_keep.h"
#include "fxjs/xfa/cjx_keyusage.h"
#include "fxjs/xfa/cjx_layoutpseudomodel.h"
#include "fxjs/xfa/cjx_line.h"
#include "fxjs/xfa/cjx_linear.h"
#include "fxjs/xfa/cjx_logpseudomodel.h"
#include "fxjs/xfa/cjx_manifest.h"
#include "fxjs/xfa/cjx_map.h"
#include "fxjs/xfa/cjx_margin.h"
#include "fxjs/xfa/cjx_mdp.h"
#include "fxjs/xfa/cjx_medium.h"
#include "fxjs/xfa/cjx_message.h"
#include "fxjs/xfa/cjx_node.h"
#include "fxjs/xfa/cjx_numericedit.h"
#include "fxjs/xfa/cjx_occur.h"
#include "fxjs/xfa/cjx_oid.h"
#include "fxjs/xfa/cjx_oids.h"
#include "fxjs/xfa/cjx_operation.h"
#include "fxjs/xfa/cjx_overflow.h"
#include "fxjs/xfa/cjx_packet.h"
#include "fxjs/xfa/cjx_pagearea.h"
#include "fxjs/xfa/cjx_pageset.h"
#include "fxjs/xfa/cjx_para.h"
#include "fxjs/xfa/cjx_password.h"
#include "fxjs/xfa/cjx_passwordedit.h"
#include "fxjs/xfa/cjx_pattern.h"
#include "fxjs/xfa/cjx_picture.h"
#include "fxjs/xfa/cjx_query.h"
#include "fxjs/xfa/cjx_radial.h"
#include "fxjs/xfa/cjx_reason.h"
#include "fxjs/xfa/cjx_reasons.h"
#include "fxjs/xfa/cjx_recordset.h"
#include "fxjs/xfa/cjx_rectangle.h"
#include "fxjs/xfa/cjx_ref.h"
#include "fxjs/xfa/cjx_rootelement.h"
#include "fxjs/xfa/cjx_script.h"
#include "fxjs/xfa/cjx_select.h"
#include "fxjs/xfa/cjx_setproperty.h"
#include "fxjs/xfa/cjx_signature.h"
#include "fxjs/xfa/cjx_signatureproperties.h"
#include "fxjs/xfa/cjx_signaturepseudomodel.h"
#include "fxjs/xfa/cjx_signdata.h"
#include "fxjs/xfa/cjx_signing.h"
#include "fxjs/xfa/cjx_soapaction.h"
#include "fxjs/xfa/cjx_soapaddress.h"
#include "fxjs/xfa/cjx_solid.h"
#include "fxjs/xfa/cjx_source.h"
#include "fxjs/xfa/cjx_sourceset.h"
#include "fxjs/xfa/cjx_speak.h"
#include "fxjs/xfa/cjx_stipple.h"
#include "fxjs/xfa/cjx_subform.h"
#include "fxjs/xfa/cjx_subformset.h"
#include "fxjs/xfa/cjx_subjectdn.h"
#include "fxjs/xfa/cjx_subjectdns.h"
#include "fxjs/xfa/cjx_submit.h"
#include "fxjs/xfa/cjx_text.h"
#include "fxjs/xfa/cjx_textedit.h"
#include "fxjs/xfa/cjx_time.h"
#include "fxjs/xfa/cjx_timestamp.h"
#include "fxjs/xfa/cjx_tooltip.h"
#include "fxjs/xfa/cjx_traversal.h"
#include "fxjs/xfa/cjx_traverse.h"
#include "fxjs/xfa/cjx_tree.h"
#include "fxjs/xfa/cjx_treelist.h"
#include "fxjs/xfa/cjx_ui.h"
#include "fxjs/xfa/cjx_update.h"
#include "fxjs/xfa/cjx_uri.h"
#include "fxjs/xfa/cjx_user.h"
#include "fxjs/xfa/cjx_validate.h"
#include "fxjs/xfa/cjx_value.h"
#include "fxjs/xfa/cjx_variables.h"
#include "fxjs/xfa/cjx_wsdladdress.h"
#include "fxjs/xfa/cjx_wsdlconnection.h"
#include "fxjs/xfa/cjx_xfa.h"
#include "fxjs/xfa/cjx_xmlconnection.h"
#include "fxjs/xfa/cjx_xsdconnection.h"
#include "xfa/fxfa/fxfa_basic.h"

const XFA_SCRIPTHIERARCHY g_XFAScriptIndex[] = {
    {/* ps */ 0, 2, 316},
    {/* to */ 2, 2, 316},
    {/* ui */ 4, 2, 316},
    {/* recordSet */ 6, 8, 316},
    {/* subsetBelow */ 14, 4, 316},
    {/* subformSet */ 18, 5, 317},
    {/* adobeExtensionLevel */ 23, 2, 316},
    {/* typeface */ 25, 1, 316},
    {/* break */ 26, 12, 316},
    {/* fontInfo */ 38, 2, 316},
    {/* numberPattern */ 40, 1, 316},
    {/* dynamicRender */ 41, 3, 316},
    {/* printScaling */ 44, 2, 316},
    {/* checkButton */ 46, 6, 316},
    {/* datePatterns */ 52, 0, 316},
    {/* sourceSet */ 52, 2, 319},
    {/* amd */ 54, 2, 316},
    {/* arc */ 56, 6, 316},
    {/* day */ 62, 0, 316},
    {/* era */ 62, 0, 316},
    {/* jog */ 62, 2, 316},
    {/* log */ 64, 2, 316},
    {/* map */ 66, 6, 316},
    {/* mdp */ 72, 4, 316},
    {/* breakBefore */ 76, 7, 316},
    {/* oid */ 83, 2, 320},
    {/* pcl */ 85, 3, 316},
    {/* pdf */ 88, 3, 316},
    {/* ref */ 91, 2, 320},
    {/* uri */ 93, 6, 320},
    {/* xdc */ 99, 4, 316},
    {/* xdp */ 103, 2, 316},
    {/* xfa */ 105, 3, 319},
    {/* xsl */ 108, 4, 316},
    {/* zpl */ 112, 3, 316},
    {/* cache */ 115, 2, 316},
    {/* margin */ 117, 6, 316},
    {/* keyUsage */ 123, 12, 316},
    {/* exclude */ 135, 2, 316},
    {/* choiceList */ 137, 5, 316},
    {/* level */ 142, 2, 316},
    {/* labelPrinter */ 144, 3, 316},
    {/* calendarSymbols */ 147, 1, 316},
    {/* para */ 148, 14, 316},
    {/* part */ 162, 2, 316},
    {/* pdfa */ 164, 2, 316},
    {/* filter */ 166, 3, 316},
    {/* present */ 169, 2, 316},
    {/* pagination */ 171, 2, 316},
    {/* encoding */ 173, 2, 316},
    {/* event */ 175, 4, 316},
    {/* whitespace */ 179, 2, 316},
    {/* defaultUi */ 181, 2, 316},
    {/* dataModel */ 183, 0, 319},
    {/* barcode */ 183, 20, 316},
    {/* timePattern */ 203, 1, 316},
    {/* batchOutput */ 204, 3, 316},
    {/* enforce */ 207, 2, 316},
    {/* currencySymbols */ 209, 0, 316},
    {/* addSilentPrint */ 209, 2, 316},
    {/* rename */ 211, 2, 316},
    {/* operation */ 213, 4, 320},
    {/* typefaces */ 217, 0, 316},
    {/* subjectDNs */ 217, 1, 316},
    {/* issuers */ 218, 3, 316},
    {/* signaturePseudoModel */ 221, 0, 312},
    {/* wsdlConnection */ 221, 1, 316},
    {/* debug */ 222, 2, 316},
    {/* delta */ 224, 3, -1},
    {/* eraNames */ 227, 0, 316},
    {/* modifyAnnots */ 227, 2, 316},
    {/* startNode */ 229, 2, 316},
    {/* button */ 231, 3, 316},
    {/* format */ 234, 2, 316},
    {/* border */ 236, 6, 316},
    {/* area */ 242, 10, 317},
    {/* hyphenation */ 252, 9, 316},
    {/* text */ 261, 5, 318},
    {/* time */ 266, 4, 318},
    {/* type */ 270, 2, 316},
    {/* overprint */ 272, 2, 316},
    {/* certificates */ 274, 5, 316},
    {/* encryptionMethods */ 279, 3, 316},
    {/* setProperty */ 282, 2, 316},
    {/* printerName */ 284, 2, 316},
    {/* startPage */ 286, 2, 316},
    {/* pageOffset */ 288, 2, 316},
    {/* dateTime */ 290, 4, 316},
    {/* comb */ 294, 3, 316},
    {/* pattern */ 297, 3, 316},
    {/* ifEmpty */ 300, 2, 316},
    {/* suppressBanner */ 302, 2, 316},
    {/* outputBin */ 304, 2, 316},
    {/* field */ 306, 36, 317},
    {/* agent */ 342, 3, 316},
    {/* outputXSL */ 345, 2, 316},
    {/* adjustData */ 347, 2, 316},
    {/* autoSave */ 349, 2, 316},
    {/* contentArea */ 351, 7, 317},
    {/* eventPseudoModel */ 358, 17, 312},
    {/* wsdlAddress */ 375, 2, 320},
    {/* solid */ 377, 2, 316},
    {/* dateTimeSymbols */ 379, 0, 316},
    {/* encryptionLevel */ 379, 2, 316},
    {/* edge */ 381, 6, 316},
    {/* stipple */ 387, 3, 316},
    {/* attributes */ 390, 2, 316},
    {/* versionControl */ 392, 4, 316},
    {/* meridiem */ 396, 0, 316},
    {/* exclGroup */ 396, 30, 316},
    {/* toolTip */ 426, 2, 320},
    {/* compress */ 428, 3, 316},
    {/* reason */ 431, 2, 320},
    {/* execute */ 433, 5, 316},
    {/* contentCopy */ 438, 2, 316},
    {/* dateTimeEdit */ 440, 3, 316},
    {/* config */ 443, 2, 316},
    {/* image */ 445, 8, 316},
    {/* #xHTML */ 453, 1, 316},
    {/* numberOfCopies */ 454, 2, 316},
    {/* behaviorOverride */ 456, 2, 316},
    {/* timeStamp */ 458, 4, 316},
    {/* month */ 462, 0, 316},
    {/* viewerPreferences */ 462, 2, 316},
    {/* scriptModel */ 464, 2, 316},
    {/* decimal */ 466, 6, 318},
    {/* subform */ 472, 31, 317},
    {/* select */ 503, 2, 320},
    {/* window */ 505, 2, 316},
    {/* localeSet */ 507, 2, 316},
    {/* handler */ 509, 4, 320},
    {/* hostPseudoModel */ 513, 11, 312},
    {/* presence */ 524, 2, 316},
    {/* record */ 526, 2, 316},
    {/* embed */ 528, 2, 316},
    {/* version */ 530, 2, 316},
    {/* command */ 532, 3, 316},
    {/* copies */ 535, 2, 316},
    {/* staple */ 537, 3, 316},
    {/* submitFormat */ 540, 3, 316},
    {/* boolean */ 543, 4, 318},
    {/* message */ 547, 4, 316},
    {/* output */ 551, 2, 316},
    {/* psMap */ 553, 0, 316},
    {/* excludeNS */ 553, 2, 316},
    {/* assist */ 555, 3, 316},
    {/* picture */ 558, 6, 316},
    {/* traversal */ 564, 2, 316},
    {/* silentPrint */ 566, 2, 316},
    {/* webClient */ 568, 3, 316},
    {/* layoutPseudoModel */ 571, 1, 312},
    {/* producer */ 572, 2, 316},
    {/* corner */ 574, 8, 316},
    {/* msgId */ 582, 2, 316},
    {/* color */ 584, 4, 316},
    {/* keep */ 588, 5, 316},
    {/* query */ 593, 3, 316},
    {/* insert */ 596, 2, 320},
    {/* imageEdit */ 598, 3, 316},
    {/* validate */ 601, 7, 316},
    {/* digestMethods */ 608, 3, 316},
    {/* numberPatterns */ 611, 0, 316},
    {/* pageSet */ 611, 4, 317},
    {/* integer */ 615, 4, 318},
    {/* soapAddress */ 619, 2, 320},
    {/* equate */ 621, 5, 316},
    {/* formFieldFilling */ 626, 2, 316},
    {/* pageRange */ 628, 2, 316},
    {/* update */ 630, 2, 320},
    {/* connectString */ 632, 2, 320},
    {/* mode */ 634, 4, 316},
    {/* layout */ 638, 2, 316},
    {/* #xml */ 640, 1, 316},
    {/* xsdConnection */ 641, 1, 316},
    {/* traverse */ 642, 4, 316},
    {/* encodings */ 646, 3, 316},
    {/* template */ 649, 2, 319},
    {/* acrobat */ 651, 2, 316},
    {/* validationMessaging */ 653, 2, 316},
    {/* signing */ 655, 3, 316},
    {/* dataWindow */ 658, 4, 312},
    {/* script */ 662, 10, 316},
    {/* addViewerPreferences */ 672, 2, 316},
    {/* alwaysEmbed */ 674, 4, 316},
    {/* passwordEdit */ 678, 4, 316},
    {/* numericEdit */ 682, 3, 316},
    {/* encryptionMethod */ 685, 2, 316},
    {/* change */ 687, 2, 316},
    {/* pageArea */ 689, 8, 317},
    {/* submitUrl */ 697, 3, 316},
    {/* oids */ 700, 3, 316},
    {/* signature */ 703, 2, 316},
    {/* ADBE_JSConsole */ 705, 2, 316},
    {/* caption */ 707, 5, 316},
    {/* relevant */ 712, 4, 316},
    {/* flipLabel */ 716, 2, 316},
    {/* exData */ 718, 8, 318},
    {/* dayNames */ 726, 1, 316},
    {/* soapAction */ 727, 2, 320},
    {/* defaultTypeface */ 729, 3, 316},
    {/* manifest */ 732, 4, 316},
    {/* overflow */ 736, 5, 316},
    {/* linear */ 741, 3, 316},
    {/* currencySymbol */ 744, 1, 316},
    {/* delete */ 745, 2, 320},
    {/* deltas */ 747, 0, 313},
    {/* digestMethod */ 747, 2, 316},
    {/* instanceManager */ 749, 3, 316},
    {/* equateRange */ 752, 5, 316},
    {/* medium */ 757, 7, 316},
    {/* textEdit */ 764, 6, 316},
    {/* templateCache */ 770, 3, 316},
    {/* compressObjectStream */ 773, 2, 316},
    {/* dataValue */ 775, 5, 316},
    {/* accessibleContent */ 780, 2, 316},
    {/* nodeList */ 782, 0, 314},
    {/* includeXDPContent */ 782, 2, 316},
    {/* xmlConnection */ 784, 1, 316},
    {/* validateApprovalSignatures */ 785, 2, 316},
    {/* signData */ 787, 5, 316},
    {/* packets */ 792, 2, 316},
    {/* datePattern */ 794, 1, 316},
    {/* duplexOption */ 795, 2, 316},
    {/* base */ 797, 2, 316},
    {/* bind */ 799, 6, 316},
    {/* compression */ 805, 2, 316},
    {/* user */ 807, 2, 320},
    {/* rectangle */ 809, 3, 316},
    {/* effectiveOutputPolicy */ 812, 4, 316},
    {/* ADBE_JSDebugger */ 816, 2, 316},
    {/* acrobat7 */ 818, 2, 316},
    {/* interactive */ 820, 2, 316},
    {/* locale */ 822, 2, 316},
    {/* currentPage */ 824, 2, 316},
    {/* data */ 826, 2, 316},
    {/* date */ 828, 4, 318},
    {/* desc */ 832, 2, 316},
    {/* encrypt */ 834, 5, 316},
    {/* draw */ 839, 20, 317},
    {/* encryption */ 859, 2, 316},
    {/* meridiemNames */ 861, 0, 316},
    {/* messaging */ 861, 2, 316},
    {/* speak */ 863, 4, 320},
    {/* dataGroup */ 867, 0, 316},
    {/* common */ 867, 2, 316},
    {/* #text */ 869, 1, 316},
    {/* paginationOverride */ 870, 2, 316},
    {/* reasons */ 872, 3, 316},
    {/* signatureProperties */ 875, 2, 316},
    {/* threshold */ 877, 2, 316},
    {/* appearanceFilter */ 879, 4, 316},
    {/* fill */ 883, 3, 316},
    {/* font */ 886, 17, 316},
    {/* form */ 903, 1, 319},
    {/* mediumInfo */ 904, 2, 316},
    {/* certificate */ 906, 2, 320},
    {/* password */ 908, 2, 320},
    {/* runScripts */ 910, 2, 316},
    {/* trace */ 912, 2, 316},
    {/* float */ 914, 4, 318},
    {/* renderPolicy */ 918, 2, 316},
    {/* logPseudoModel */ 920, 0, 312},
    {/* destination */ 920, 2, 316},
    {/* value */ 922, 4, 316},
    {/* bookend */ 926, 4, 316},
    {/* exObject */ 930, 6, 316},
    {/* openAction */ 936, 2, 316},
    {/* neverEmbed */ 938, 4, 316},
    {/* bindItems */ 942, 3, 316},
    {/* calculate */ 945, 3, 316},
    {/* print */ 948, 2, 316},
    {/* extras */ 950, 3, 316},
    {/* proto */ 953, 0, 316},
    {/* dSigData */ 953, 0, 316},
    {/* creator */ 953, 2, 316},
    {/* connect */ 955, 7, 316},
    {/* permissions */ 962, 2, 316},
    {/* connectionSet */ 964, 0, 319},
    {/* submit */ 964, 7, 316},
    {/* range */ 971, 2, 316},
    {/* linearized */ 973, 2, 316},
    {/* packet */ 975, 1, 316},
    {/* rootElement */ 976, 2, 320},
    {/* plaintextMetadata */ 978, 4, 316},
    {/* numberSymbols */ 982, 0, 316},
    {/* printHighQuality */ 982, 2, 316},
    {/* driver */ 984, 2, 316},
    {/* incrementalLoad */ 986, 4, 316},
    {/* subjectDN */ 990, 1, 316},
    {/* compressLogicalStructure */ 991, 2, 316},
    {/* incrementalMerge */ 993, 2, 316},
    {/* radial */ 995, 3, 316},
    {/* variables */ 998, 2, 317},
    {/* timePatterns */ 1000, 0, 316},
    {/* effectiveInputPolicy */ 1000, 4, 316},
    {/* nameAttr */ 1004, 4, 316},
    {/* conformance */ 1008, 2, 316},
    {/* transform */ 1010, 3, 316},
    {/* lockDocument */ 1013, 4, 316},
    {/* breakAfter */ 1017, 7, 316},
    {/* line */ 1024, 4, 316},
    {/* list */ 1028, 1, 313},
    {/* source */ 1029, 3, 316},
    {/* occur */ 1032, 5, 316},
    {/* pickTrayByPDFSize */ 1037, 2, 316},
    {/* monthNames */ 1039, 1, 316},
    {/* severity */ 1040, 4, 316},
    {/* groupParent */ 1044, 2, 316},
    {/* documentAssembly */ 1046, 2, 316},
    {/* numberSymbol */ 1048, 1, 316},
    {/* tagged */ 1049, 2, 316},
    {/*  */ 1051, 5, 316},
    {/*  */ 1056, 1, -1},
    {/*  */ 1057, 1, 312},
    {/*  */ 1058, 0, 313},
    {/*  */ 1058, 8, 312},
    {/*  */ 1066, 6, 315},
    {/*  */ 1072, 0, 316},
    {/*  */ 1072, 0, 316},
    {/*  */ 1072, 2, 316},
    {/*  */ 1074, 2, 316}};
const size_t g_szScriptIndexCount = FX_ArraySize(g_XFAScriptIndex);

#undef ATTR
#define ATTR(a, b, c, d, e) a, d, e, reinterpret_cast<XFA_ATTRIBUTE_CALLBACK>(c)

const XFA_SCRIPTATTRIBUTEINFO g_SomAttributeData[] = {
    /* ps */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* to */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* ui */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Ui::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Ui::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* recordSet */
    {ATTR(0xb3543a6,
          L"max",
          &CJX_RecordSet::max,
          XFA_Attribute::Max,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_RecordSet::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x45a6daf8,
          L"eofAction",
          &CJX_RecordSet::eofAction,
          XFA_Attribute::EofAction,
          XFA_ScriptType::Basic)},
    {ATTR(0x5ec958c0,
          L"cursorType",
          &CJX_RecordSet::cursorType,
          XFA_Attribute::CursorType,
          XFA_ScriptType::Basic)},
    {ATTR(0x79975f2b,
          L"lockType",
          &CJX_RecordSet::lockType,
          XFA_Attribute::LockType,
          XFA_ScriptType::Basic)},
    {ATTR(0xa5340ff5,
          L"bofAction",
          &CJX_RecordSet::bofAction,
          XFA_Attribute::BofAction,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_RecordSet::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc5762157,
          L"cursorLocation",
          &CJX_RecordSet::cursorLocation,
          XFA_Attribute::CursorLocation,
          XFA_ScriptType::Basic)},

    /* subsetBelow */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* subformSet */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_SubformSet::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1ee2d24d,
          L"instanceIndex",
          &CJX_SubformSet::instanceIndex,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x8c99377e,
          L"relation",
          &CJX_SubformSet::relation,
          XFA_Attribute::Relation,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_SubformSet::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_SubformSet::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* adobeExtensionLevel */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* typeface */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* break */
    {ATTR(0x3106c3a,
          L"beforeTarget",
          &CJX_Break::beforeTarget,
          XFA_Attribute::BeforeTarget,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Break::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x13a08bdb,
          L"overflowTarget",
          &CJX_Break::overflowTarget,
          XFA_Attribute::OverflowTarget,
          XFA_ScriptType::Basic)},
    {ATTR(0x169134a1,
          L"overflowLeader",
          &CJX_Break::overflowLeader,
          XFA_Attribute::OverflowLeader,
          XFA_ScriptType::Basic)},
    {ATTR(0x20914367,
          L"overflowTrailer",
          &CJX_Break::overflowTrailer,
          XFA_Attribute::OverflowTrailer,
          XFA_ScriptType::Basic)},
    {ATTR(0x453eaf38,
          L"startNew",
          &CJX_Break::startNew,
          XFA_Attribute::StartNew,
          XFA_ScriptType::Basic)},
    {ATTR(0x64110ab5,
          L"bookendTrailer",
          &CJX_Break::bookendTrailer,
          XFA_Attribute::BookendTrailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xb6b44172,
          L"after",
          &CJX_Break::after,
          XFA_Attribute::After,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Break::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc3c1442f,
          L"bookendLeader",
          &CJX_Break::bookendLeader,
          XFA_Attribute::BookendLeader,
          XFA_ScriptType::Basic)},
    {ATTR(0xcb150479,
          L"afterTarget",
          &CJX_Break::afterTarget,
          XFA_Attribute::AfterTarget,
          XFA_ScriptType::Basic)},
    {ATTR(0xf4ffce73,
          L"before",
          &CJX_Break::before,
          XFA_Attribute::Before,
          XFA_ScriptType::Basic)},

    /* fontInfo */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* numberPattern */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* dynamicRender */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Object::Script_Som_DefaultValue_Read,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* printScaling */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* checkButton */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_CheckButton::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x47cfa43a,
          L"allowNeutral",
          &CJX_CheckButton::allowNeutral,
          XFA_Attribute::AllowNeutral,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2fd80b,
          L"mark",
          &CJX_CheckButton::mark,
          XFA_Attribute::Mark,
          XFA_ScriptType::Basic)},
    {ATTR(0x8ed182d1,
          L"shape",
          &CJX_CheckButton::shape,
          XFA_Attribute::Shape,
          XFA_ScriptType::Basic)},
    {ATTR(0xa686975b,
          L"size",
          &CJX_CheckButton::size,
          XFA_Attribute::Size,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_CheckButton::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* datePatterns */

    /* sourceSet */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_SourceSet::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_SourceSet::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* amd */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* arc */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Arc::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5c054755,
          L"startAngle",
          &CJX_Arc::startAngle,
          XFA_Attribute::StartAngle,
          XFA_ScriptType::Basic)},
    {ATTR(0x74788f8b,
          L"sweepAngle",
          &CJX_Arc::sweepAngle,
          XFA_Attribute::SweepAngle,
          XFA_ScriptType::Basic)},
    {ATTR(0x9d833d75,
          L"circular",
          &CJX_Arc::circular,
          XFA_Attribute::Circular,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Arc::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          L"hand",
          &CJX_Arc::hand,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* day */

    /* era */

    /* jog */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* log */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* map */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Map::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xb0e5485d,
          L"bind",
          &CJX_Map::bind,
          XFA_Attribute::Bind,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Map::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xcd7f7b54,
          L"from",
          &CJX_Map::from,
          XFA_Attribute::From,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* mdp */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Mdp::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e29d794,
          L"signatureType",
          &CJX_Mdp::signatureType,
          XFA_Attribute::SignatureType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Mdp::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe11a2cbc,
          L"permissions",
          &CJX_Mdp::permissions,
          XFA_Attribute::Permissions,
          XFA_ScriptType::Basic)},

    /* breakBefore */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_BreakBefore::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x453eaf38,
          L"startNew",
          &CJX_BreakBefore::startNew,
          XFA_Attribute::StartNew,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          L"trailer",
          &CJX_BreakBefore::trailer,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xa6118c89,
          L"targetType",
          &CJX_BreakBefore::targetType,
          XFA_Attribute::TargetType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_BreakBefore::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_BreakBefore::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          L"leader",
          &CJX_BreakBefore::leader,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* oid */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Oid::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Oid::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* pcl */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pdf */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* ref */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Ref::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Ref::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* uri */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Uri::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Uri::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xdc */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xdp */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xfa */
    {ATTR(0x2d574d58,
          L"this",
          &CJX_Xfa::thisValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0x4fdc3454,
          L"timeStamp",
          &CJX_Xfa::timeStamp,
          XFA_Attribute::TimeStamp,
          XFA_ScriptType::Basic)},
    {ATTR(0xb598a1f7,
          L"uuid",
          &CJX_Xfa::uuid,
          XFA_Attribute::Uuid,
          XFA_ScriptType::Basic)},

    /* xsl */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* zpl */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* cache */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* margin */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Margin::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xcfea02e,
          L"leftInset",
          &CJX_Margin::leftInset,
          XFA_Attribute::LeftInset,
          XFA_ScriptType::Basic)},
    {ATTR(0x1356caf8,
          L"bottomInset",
          &CJX_Margin::bottomInset,
          XFA_Attribute::BottomInset,
          XFA_ScriptType::Basic)},
    {ATTR(0x25764436,
          L"topInset",
          &CJX_Margin::topInset,
          XFA_Attribute::TopInset,
          XFA_ScriptType::Basic)},
    {ATTR(0x8a692521,
          L"rightInset",
          &CJX_Margin::rightInset,
          XFA_Attribute::RightInset,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Margin::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* keyUsage */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_KeyUsage::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1e459b8f,
          L"nonRepudiation",
          &CJX_KeyUsage::nonRepudiation,
          XFA_Attribute::NonRepudiation,
          XFA_ScriptType::Basic)},
    {ATTR(0x2bb3f470,
          L"encipherOnly",
          &CJX_KeyUsage::encipherOnly,
          XFA_Attribute::EncipherOnly,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_KeyUsage::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x5f760b50,
          L"digitalSignature",
          &CJX_KeyUsage::digitalSignature,
          XFA_Attribute::DigitalSignature,
          XFA_ScriptType::Basic)},
    {ATTR(0x69aa2292,
          L"crlSign",
          &CJX_KeyUsage::crlSign,
          XFA_Attribute::CrlSign,
          XFA_ScriptType::Basic)},
    {ATTR(0x98fd4d81,
          L"keyAgreement",
          &CJX_KeyUsage::keyAgreement,
          XFA_Attribute::KeyAgreement,
          XFA_ScriptType::Basic)},
    {ATTR(0xa66404cb,
          L"keyEncipherment",
          &CJX_KeyUsage::keyEncipherment,
          XFA_Attribute::KeyEncipherment,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_KeyUsage::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xca5dc27c,
          L"dataEncipherment",
          &CJX_KeyUsage::dataEncipherment,
          XFA_Attribute::DataEncipherment,
          XFA_ScriptType::Basic)},
    {ATTR(0xe8f118a8,
          L"keyCertSign",
          &CJX_KeyUsage::keyCertSign,
          XFA_Attribute::KeyCertSign,
          XFA_ScriptType::Basic)},
    {ATTR(0xfea53ec6,
          L"decipherOnly",
          &CJX_KeyUsage::decipherOnly,
          XFA_Attribute::DecipherOnly,
          XFA_ScriptType::Basic)},

    /* exclude */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* choiceList */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ChoiceList::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8b90e1f2,
          L"open",
          &CJX_ChoiceList::open,
          XFA_Attribute::Open,
          XFA_ScriptType::Basic)},
    {ATTR(0x957fa006,
          L"commitOn",
          &CJX_ChoiceList::commitOn,
          XFA_Attribute::CommitOn,
          XFA_ScriptType::Basic)},
    {ATTR(0xb12128b7,
          L"textEntry",
          &CJX_ChoiceList::textEntry,
          XFA_Attribute::TextEntry,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ChoiceList::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* level */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* labelPrinter */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* calendarSymbols */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* para */
    {ATTR(0x2282c73,
          L"hAlign",
          &CJX_Para::hAlign,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x8d4f1c7,
          L"textIndent",
          &CJX_Para::textIndent,
          XFA_Attribute::TextIndent,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Para::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2a82d99c,
          L"marginRight",
          &CJX_Para::marginRight,
          XFA_Attribute::MarginRight,
          XFA_ScriptType::Basic)},
    {ATTR(0x534729c9,
          L"marginLeft",
          &CJX_Para::marginLeft,
          XFA_Attribute::MarginLeft,
          XFA_ScriptType::Basic)},
    {ATTR(0x5739d1ff,
          L"radixOffset",
          &CJX_Para::radixOffset,
          XFA_Attribute::RadixOffset,
          XFA_ScriptType::Basic)},
    {ATTR(0x577682ac,
          L"preserve",
          &CJX_Para::preserve,
          XFA_Attribute::Preserve,
          XFA_ScriptType::Basic)},
    {ATTR(0x731e0665,
          L"spaceBelow",
          &CJX_Para::spaceBelow,
          XFA_Attribute::SpaceBelow,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          L"vAlign",
          &CJX_Para::vAlign,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x836d4d7c,
          L"tabDefault",
          &CJX_Para::tabDefault,
          XFA_Attribute::TabDefault,
          XFA_ScriptType::Basic)},
    {ATTR(0x8fa01790,
          L"tabStops",
          &CJX_Para::tabStops,
          XFA_Attribute::TabStops,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Para::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd4b01921,
          L"lineHeight",
          &CJX_Para::lineHeight,
          XFA_Attribute::LineHeight,
          XFA_ScriptType::Basic)},
    {ATTR(0xe18b5659,
          L"spaceAbove",
          &CJX_Para::spaceAbove,
          XFA_Attribute::SpaceAbove,
          XFA_ScriptType::Basic)},

    /* part */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pdfa */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* filter */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Filter::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Filter::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd861f8af,
          L"addRevocationInfo",
          &CJX_Filter::addRevocationInfo,
          XFA_Attribute::AddRevocationInfo,
          XFA_ScriptType::Basic)},

    /* present */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pagination */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* encoding */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Encoding::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Encoding::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* event */
    {ATTR(0xbb8df5d,
          L"ref",
          &CJX_Event::ref,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Event::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Event::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6128d8d,
          L"activity",
          &CJX_Event::activity,
          XFA_Attribute::Activity,
          XFA_ScriptType::Basic)},

    /* whitespace */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* defaultUi */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_DefaultUi::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_DefaultUi::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* dataModel */

    /* barcode */
    {ATTR(0x43e349b,
          L"dataRowCount",
          &CJX_Barcode::dataRowCount,
          XFA_Attribute::DataRowCount,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Barcode::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x28e17e91,
          L"dataPrep",
          &CJX_Barcode::dataPrep,
          XFA_Attribute::DataPrep,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Barcode::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x3650557e,
          L"textLocation",
          &CJX_Barcode::textLocation,
          XFA_Attribute::TextLocation,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b582286,
          L"moduleWidth",
          &CJX_Barcode::moduleWidth,
          XFA_Attribute::ModuleWidth,
          XFA_ScriptType::Basic)},
    {ATTR(0x52666f1c,
          L"printCheckDigit",
          &CJX_Barcode::printCheckDigit,
          XFA_Attribute::PrintCheckDigit,
          XFA_ScriptType::Basic)},
    {ATTR(0x5404d6df,
          L"moduleHeight",
          &CJX_Barcode::moduleHeight,
          XFA_Attribute::ModuleHeight,
          XFA_ScriptType::Basic)},
    {ATTR(0x5ab23b6c,
          L"startChar",
          &CJX_Barcode::startChar,
          XFA_Attribute::StartChar,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c732a66,
          L"truncate",
          &CJX_Barcode::truncate,
          XFA_Attribute::Truncate,
          XFA_ScriptType::Basic)},
    {ATTR(0x8d181d61,
          L"wideNarrowRatio",
          &CJX_Barcode::wideNarrowRatio,
          XFA_Attribute::WideNarrowRatio,
          XFA_ScriptType::Basic)},
    {ATTR(0x99800d7a,
          L"errorCorrectionLevel",
          &CJX_Barcode::errorCorrectionLevel,
          XFA_Attribute::ErrorCorrectionLevel,
          XFA_ScriptType::Basic)},
    {ATTR(0x9a63da3d,
          L"upsMode",
          &CJX_Barcode::upsMode,
          XFA_Attribute::UpsMode,
          XFA_ScriptType::Basic)},
    {ATTR(0xaf754613,
          L"checksum",
          &CJX_Barcode::checksum,
          XFA_Attribute::Checksum,
          XFA_ScriptType::Basic)},
    {ATTR(0xb045fbc5,
          L"charEncoding",
          &CJX_Barcode::charEncoding,
          XFA_Attribute::CharEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Barcode::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc035c6b1,
          L"dataColumnCount",
          &CJX_Barcode::dataColumnCount,
          XFA_Attribute::DataColumnCount,
          XFA_ScriptType::Basic)},
    {ATTR(0xd3c84d25,
          L"rowColumnRatio",
          &CJX_Barcode::rowColumnRatio,
          XFA_Attribute::RowColumnRatio,
          XFA_ScriptType::Basic)},
    {ATTR(0xd57c513c,
          L"dataLength",
          &CJX_Barcode::dataLength,
          XFA_Attribute::DataLength,
          XFA_ScriptType::Basic)},
    {ATTR(0xf575ca75,
          L"endChar",
          &CJX_Barcode::endChar,
          XFA_Attribute::EndChar,
          XFA_ScriptType::Basic)},

    /* timePattern */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* batchOutput */
    {ATTR(0x28dee6e9,
          L"format",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Format,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* enforce */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* currencySymbols */

    /* addSilentPrint */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* rename */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* operation */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Operation::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x60d4c8b1,
          L"output",
          &CJX_Operation::output,
          XFA_Attribute::Output,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Operation::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6a39990,
          L"input",
          &CJX_Operation::input,
          XFA_Attribute::Input,
          XFA_ScriptType::Basic)},

    /* typefaces */

    /* subjectDNs */
    {ATTR(0x2f16a382,
          L"type",
          &CJX_SubjectDNs::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},

    /* issuers */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Issuers::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Issuers::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Issuers::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* signaturePseudoModel */

    /* wsdlConnection */
    {ATTR(0x2b5df51e,
          L"dataDescription",
          &CJX_WsdlConnection::dataDescription,
          XFA_Attribute::DataDescription,
          XFA_ScriptType::Basic)},

    /* debug */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* delta */
    {ATTR(0x6c0d9600,
          L"currentValue",
          &CJX_Delta::currentValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x942643f0,
          L"savedValue",
          &CJX_Delta::savedValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_Delta::target,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* eraNames */

    /* modifyAnnots */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* startNode */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* button */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Button::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Button::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd4cc53f8,
          L"highlight",
          &CJX_Button::highlight,
          XFA_Attribute::Highlight,
          XFA_ScriptType::Basic)},

    /* format */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Format::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Format::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* border */
    {ATTR(0x5518c25,
          L"break",
          &CJX_Border::breakValue,
          XFA_Attribute::Break,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Border::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Border::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_Border::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Border::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          L"hand",
          &CJX_Border::hand,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* area */
    {ATTR(0x78, L"x", &CJX_Area::x, XFA_Attribute::X, XFA_ScriptType::Basic)},
    {ATTR(0x79, L"y", &CJX_Area::y, XFA_Attribute::Y, XFA_ScriptType::Basic)},
    {ATTR(0x21aed,
          L"id",
          &CJX_Area::id,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Area::name,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Area::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1059ec18,
          L"level",
          &CJX_Object::Script_Attribute_Integer,
          XFA_Attribute::Level,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_Area::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          L"colSpan",
          &CJX_Area::colSpan,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Area::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* hyphenation */
    {ATTR(0x21aed,
          L"id",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f105f72,
          L"wordCharacterCount",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::WordCharacterCount,
          XFA_ScriptType::Basic)},
    {ATTR(0x3d123c26,
          L"hyphenate",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Hyphenate,
          XFA_ScriptType::Basic)},
    {ATTR(0x66539c48,
          L"excludeInitialCap",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::ExcludeInitialCap,
          XFA_ScriptType::Basic)},
    {ATTR(0x6a95c976,
          L"pushCharacterCount",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::PushCharacterCount,
          XFA_ScriptType::Basic)},
    {ATTR(0x982bd892,
          L"remainCharacterCount",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::RemainCharacterCount,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe5c96d6a,
          L"excludeAllCaps",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::ExcludeAllCaps,
          XFA_ScriptType::Basic)},

    /* text */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Text::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8af2e657,
          L"maxChars",
          &CJX_Text::maxChars,
          XFA_Attribute::MaxChars,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Text::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Text::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Text::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* time */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Time::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Time::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Time::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Time::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* type */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* overprint */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* certificates */
    {ATTR(0xc080cd3,
          L"url",
          &CJX_Certificates::url,
          XFA_Attribute::Url,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Certificates::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa6710262,
          L"credentialServerPolicy",
          &CJX_Certificates::credentialServerPolicy,
          XFA_Attribute::CredentialServerPolicy,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Certificates::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2ba0923,
          L"urlPolicy",
          &CJX_Certificates::urlPolicy,
          XFA_Attribute::UrlPolicy,
          XFA_ScriptType::Basic)},

    /* encryptionMethods */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* setProperty */
    {ATTR(0x47d03490,
          L"connection",
          &CJX_SetProperty::connection,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_SetProperty::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},

    /* printerName */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* startPage */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pageOffset */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* dateTime */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_DateTime::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_DateTime::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_DateTime::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_DateTime::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* comb */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Comb::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x78bff531,
          L"numberOfCells",
          &CJX_Comb::numberOfCells,
          XFA_Attribute::NumberOfCells,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Comb::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* pattern */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Pattern::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Pattern::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Pattern::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* ifEmpty */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* suppressBanner */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* outputBin */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* field */
    {ATTR(0x68, L"h", &CJX_Field::h, XFA_Attribute::H, XFA_ScriptType::Basic)},
    {ATTR(0x77, L"w", &CJX_Field::w, XFA_Attribute::W, XFA_ScriptType::Basic)},
    {ATTR(0x78, L"x", &CJX_Field::x, XFA_Attribute::X, XFA_ScriptType::Basic)},
    {ATTR(0x79, L"y", &CJX_Field::y, XFA_Attribute::Y, XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          L"hAlign",
          &CJX_Field::hAlign,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Field::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1abbd7e0,
          L"dataNode",
          &CJX_Object::Script_Som_DataNode,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0x25839852,
          L"access",
          &CJX_Field::access,
          XFA_Attribute::Access,
          XFA_ScriptType::Basic)},
    {ATTR(0x2ee7678f,
          L"rotate",
          &CJX_Field::rotate,
          XFA_Attribute::Rotate,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b1ddd06,
          L"fillColor",
          &CJX_Field::fillColor,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x54c399e3,
          L"formattedValue",
          &CJX_Field::formattedValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Field::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a3b375d,
          L"borderColor",
          &CJX_Field::borderColor,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x5e936ed6,
          L"fontColor",
          &CJX_Field::fontColor,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x6826c408,
          L"parentSubform",
          &CJX_Field::parentSubform,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x79b67434,
          L"mandatoryMessage",
          &CJX_Field::mandatoryMessage,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          L"vAlign",
          &CJX_Field::vAlign,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          L"maxH",
          &CJX_Field::maxH,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          L"maxW",
          &CJX_Field::maxW,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          L"minH",
          &CJX_Field::minH,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          L"minW",
          &CJX_Field::minW,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x85fd6faf,
          L"mandatory",
          &CJX_Field::mandatory,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_Field::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0x964fb42e,
          L"formatMessage",
          &CJX_Field::formatMessage,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xa03cf627,
          L"rawValue",
          &CJX_Field::rawValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Field::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xa60dd202,
          L"length",
          &CJX_Object::Script_Field_Length,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          L"colSpan",
          &CJX_Field::colSpan,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Field::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc8fa350,
          L"locale",
          &CJX_Field::locale,
          XFA_Attribute::Locale,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          L"anchorType",
          &CJX_Field::anchorType,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},
    {ATTR(0xc4fed09b,
          L"accessKey",
          &CJX_Field::accessKey,
          XFA_Attribute::AccessKey,
          XFA_ScriptType::Basic)},
    {ATTR(0xcabfa3d0,
          L"validationMessage",
          &CJX_Field::validationMessage,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xdcecd663,
          L"editValue",
          &CJX_Field::editValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xe07e5061,
          L"selectedIndex",
          &CJX_Field::selectedIndex,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xf65e34be,
          L"borderWidth",
          &CJX_Field::borderWidth,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* agent */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* outputXSL */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* adjustData */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* autoSave */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* contentArea */
    {ATTR(0x68,
          L"h",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          L"w",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          L"x",
          &CJX_ContentArea::x,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          L"y",
          &CJX_ContentArea::y,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ContentArea::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_ContentArea::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ContentArea::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* eventPseudoModel */
    {ATTR(0xd843798,
          L"fullText",
          &CJX_EventPseudoModel::fullText,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x1b6d1cf5,
          L"reenter",
          &CJX_EventPseudoModel::reenter,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x1e6ffa9a,
          L"prevContentType",
          &CJX_EventPseudoModel::prevContentType,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x25a3c206,
          L"soapFaultString",
          &CJX_EventPseudoModel::soapFaultString,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x2e00c007,
          L"newContentType",
          &CJX_EventPseudoModel::newContentType,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x4570500f,
          L"modifier",
          &CJX_EventPseudoModel::modifier,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x50e2e33b,
          L"selEnd",
          &CJX_EventPseudoModel::selEnd,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x57de87c2,
          L"prevText",
          &CJX_EventPseudoModel::prevText,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x6ea04e0a,
          L"soapFaultCode",
          &CJX_EventPseudoModel::soapFaultCode,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x6f6556cf,
          L"newText",
          &CJX_EventPseudoModel::newText,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x891f4606,
          L"change",
          &CJX_EventPseudoModel::change,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x8fa3c19e,
          L"shift",
          &CJX_EventPseudoModel::shift,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xa9d9b2e1,
          L"keyDown",
          &CJX_EventPseudoModel::keyDown,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbfc89db2,
          L"selStart",
          &CJX_EventPseudoModel::selStart,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xc32a5812,
          L"commitKey",
          &CJX_EventPseudoModel::commitKey,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_EventPseudoModel::target,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xd8624e04,
          L"cancelAction",
          &CJX_EventPseudoModel::cancelAction,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* wsdlAddress */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_WsdlAddress::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_WsdlAddress::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* solid */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Solid::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Solid::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* dateTimeSymbols */

    /* encryptionLevel */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* edge */
    {ATTR(0xa2e3514,
          L"cap",
          &CJX_Edge::cap,
          XFA_Attribute::Cap,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Edge::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5392ea58,
          L"stroke",
          &CJX_Edge::stroke,
          XFA_Attribute::Stroke,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Edge::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x94446dcc,
          L"thickness",
          &CJX_Edge::thickness,
          XFA_Attribute::Thickness,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Edge::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* stipple */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Stipple::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1ec8ab2c,
          L"rate",
          &CJX_Stipple::rate,
          XFA_Attribute::Rate,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Stipple::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* attributes */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* versionControl */
    {ATTR(0x7b29630a,
          L"sourceBelow",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::SourceBelow,
          XFA_ScriptType::Basic)},
    {ATTR(0x8fc36c0a,
          L"outputBelow",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::OutputBelow,
          XFA_ScriptType::Basic)},
    {ATTR(0xe996b2fe,
          L"sourceAbove",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::SourceAbove,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* meridiem */

    /* exclGroup */
    {ATTR(0x68,
          L"h",
          &CJX_ExclGroup::h,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          L"w",
          &CJX_ExclGroup::w,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          L"x",
          &CJX_ExclGroup::x,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          L"y",
          &CJX_ExclGroup::y,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          L"hAlign",
          &CJX_ExclGroup::hAlign,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ExclGroup::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xf23332f,
          L"errorText",
          &CJX_Object::Script_ExclGroup_ErrorText,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x1abbd7e0,
          L"dataNode",
          &CJX_Object::Script_Som_DataNode,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0x25839852,
          L"access",
          &CJX_ExclGroup::access,
          XFA_Attribute::Access,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b1ddd06,
          L"fillColor",
          &CJX_ExclGroup::fillColor,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_ExclGroup::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a3b375d,
          L"borderColor",
          &CJX_ExclGroup::borderColor,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x79b67434,
          L"mandatoryMessage",
          &CJX_ExclGroup::mandatoryMessage,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          L"vAlign",
          &CJX_ExclGroup::vAlign,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          L"maxH",
          &CJX_ExclGroup::maxH,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          L"maxW",
          &CJX_ExclGroup::maxW,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          L"minH",
          &CJX_ExclGroup::minH,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          L"minW",
          &CJX_ExclGroup::minW,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7e7e845e,
          L"layout",
          &CJX_ExclGroup::layout,
          XFA_Attribute::Layout,
          XFA_ScriptType::Basic)},
    {ATTR(0x846599f8,
          L"transient",
          &CJX_ExclGroup::transient,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x85fd6faf,
          L"mandatory",
          &CJX_ExclGroup::mandatory,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_ExclGroup::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xa03cf627,
          L"rawValue",
          &CJX_ExclGroup::rawValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_ExclGroup::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          L"colSpan",
          &CJX_ExclGroup::colSpan,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ExclGroup::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          L"anchorType",
          &CJX_ExclGroup::anchorType,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},
    {ATTR(0xc4fed09b,
          L"accessKey",
          &CJX_ExclGroup::accessKey,
          XFA_Attribute::AccessKey,
          XFA_ScriptType::Basic)},
    {ATTR(0xcabfa3d0,
          L"validationMessage",
          &CJX_ExclGroup::validationMessage,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xf65e34be,
          L"borderWidth",
          &CJX_ExclGroup::borderWidth,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* toolTip */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ToolTip::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ToolTip::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* compress */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xeda9017a,
          L"scope",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Scope,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* reason */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Reason::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Reason::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* execute */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Execute::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x47d03490,
          L"connection",
          &CJX_Execute::connection,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0x6cfa828a,
          L"runAt",
          &CJX_Execute::runAt,
          XFA_Attribute::RunAt,
          XFA_ScriptType::Basic)},
    {ATTR(0xa1b0d2f5,
          L"executeType",
          &CJX_Execute::executeType,
          XFA_Attribute::ExecuteType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Execute::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* contentCopy */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* dateTimeEdit */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_DateTimeEdit::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_DateTimeEdit::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          L"hScrollPolicy",
          &CJX_DateTimeEdit::hScrollPolicy,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* config */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* image */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Image::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          L"contentType",
          &CJX_Image::contentType,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x54fa722c,
          L"transferEncoding",
          &CJX_Image::transferEncoding,
          XFA_Attribute::TransferEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Image::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Image::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd171b240,
          L"aspect",
          &CJX_Image::aspect,
          XFA_Attribute::Aspect,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Image::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xdb55fec5,
          L"href",
          &CJX_Image::href,
          XFA_Attribute::Href,
          XFA_ScriptType::Basic)},

    /* #xHTML */
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* numberOfCopies */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* behaviorOverride */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* timeStamp */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_TimeStamp::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_TimeStamp::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x7f6fd3d7,
          L"server",
          &CJX_TimeStamp::server,
          XFA_Attribute::Server,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_TimeStamp::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* month */

    /* viewerPreferences */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* scriptModel */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* decimal */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Decimal::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x4b8bc840,
          L"fracDigits",
          &CJX_Decimal::fracDigits,
          XFA_Attribute::FracDigits,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Decimal::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Decimal::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Decimal::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xde7f92ba,
          L"leadDigits",
          &CJX_Decimal::leadDigits,
          XFA_Attribute::LeadDigits,
          XFA_ScriptType::Basic)},

    /* subform */
    {ATTR(0x68,
          L"h",
          &CJX_Subform::h,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          L"w",
          &CJX_Subform::w,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          L"x",
          &CJX_Subform::x,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          L"y",
          &CJX_Subform::y,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          L"hAlign",
          &CJX_Subform::hAlign,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Subform::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1414d431,
          L"allowMacro",
          &CJX_Subform::allowMacro,
          XFA_Attribute::AllowMacro,
          XFA_ScriptType::Basic)},
    {ATTR(0x1517dfa1,
          L"columnWidths",
          &CJX_Subform::columnWidths,
          XFA_Attribute::ColumnWidths,
          XFA_ScriptType::Basic)},
    {ATTR(0x1abbd7e0,
          L"dataNode",
          &CJX_Object::Script_Som_DataNode,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0x1ee2d24d,
          L"instanceIndex",
          &CJX_Subform::instanceIndex,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x25839852,
          L"access",
          &CJX_Object::Script_Attribute_String,

          XFA_Attribute::Access,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b1ddd06,
          L"fillColor",
          &CJX_Object::Script_Som_FillColor,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Subform::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a3b375d,
          L"borderColor",
          &CJX_Object::Script_Som_BorderColor,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          L"vAlign",
          &CJX_Subform::vAlign,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          L"maxH",
          &CJX_Subform::maxH,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          L"maxW",
          &CJX_Subform::maxW,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          L"minH",
          &CJX_Subform::minH,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          L"minW",
          &CJX_Subform::minW,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7e7e845e,
          L"layout",
          &CJX_Subform::layout,
          XFA_Attribute::Layout,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_Subform::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0x9cc17d75,
          L"mergeMode",
          &CJX_Object::Script_Attribute_String,

          XFA_Attribute::MergeMode,
          XFA_ScriptType::Basic)},
    {ATTR(0x9f3e9510,
          L"instanceManager",
          &CJX_Object::Script_Subform_InstanceManager,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0xac06e2b0,
          L"colSpan",
          &CJX_Subform::colSpan,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Subform::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc8fa350,
          L"locale",
          &CJX_Subform::locale,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          L"anchorType",
          &CJX_Subform::anchorType,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},
    {ATTR(0xcabfa3d0,
          L"validationMessage",
          &CJX_Subform::validationMessage,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xe4c3a5e5,
          L"restoreState",
          &CJX_Subform::restoreState,
          XFA_Attribute::RestoreState,
          XFA_ScriptType::Basic)},
    {ATTR(0xeda9017a,
          L"scope",
          &CJX_Subform::scope,
          XFA_Attribute::Scope,
          XFA_ScriptType::Basic)},
    {ATTR(0xf65e34be,
          L"borderWidth",
          &CJX_Object::Script_Som_BorderWidth,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* select */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Select::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Select::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* window */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* localeSet */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* handler */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Handler::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Handler::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a50e9e6,
          L"version",
          &CJX_Handler::version,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Handler::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* hostPseudoModel */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_HostPseudoModel::name,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x66c1ae9,
          L"validationsEnabled",
          &CJX_HostPseudoModel::validationsEnabled,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x14d04502,
          L"title",
          &CJX_HostPseudoModel::title,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x392ae445,
          L"platform",
          &CJX_HostPseudoModel::platform,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a50e9e6,
          L"version",
          &CJX_HostPseudoModel::version,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x66cb1eed,
          L"variation",
          &CJX_HostPseudoModel::variation,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x7717cbc4,
          L"language",
          &CJX_HostPseudoModel::language,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x86698963,
          L"appType",
          &CJX_HostPseudoModel::appType,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x94ff9e8d,
          L"calculationsEnabled",
          &CJX_HostPseudoModel::calculationsEnabled,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbcd44940,
          L"currentPage",
          &CJX_HostPseudoModel::currentPage,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xd592b920,
          L"numPages",
          &CJX_HostPseudoModel::numPages,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* presence */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* record */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* embed */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* version */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* command */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Command::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x24d85167,
          L"timeout",
          &CJX_Command::timeout,
          XFA_Attribute::Timeout,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Command::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* copies */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* staple */
    {ATTR(0x7d9fd7c5,
          L"mode",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Mode,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* submitFormat */
    {ATTR(0x7d9fd7c5,
          L"mode",
          &CJX_Object::Script_SubmitFormat_Mode,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* boolean */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Boolean::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Boolean::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Boolean::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Boolean::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* message */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Message::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Message::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* output */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* psMap */

    /* excludeNS */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* assist */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Assist::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2038c9b2,
          L"role",
          &CJX_Assist::role,
          XFA_Attribute::Role,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Assist::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* picture */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Picture::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Picture::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Picture::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Picture::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* traversal */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Traversal::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Traversal::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* silentPrint */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* webClient */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* layoutPseudoModel */
    {ATTR(0xfcef86b5,
          L"ready",
          &CJX_LayoutPseudoModel::ready,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* producer */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* corner */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Corner::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5392ea58,
          L"stroke",
          &CJX_Corner::stroke,
          XFA_Attribute::Stroke,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Corner::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x7b95e661,
          L"inverted",
          &CJX_Corner::inverted,
          XFA_Attribute::Inverted,
          XFA_ScriptType::Basic)},
    {ATTR(0x94446dcc,
          L"thickness",
          &CJX_Corner::thickness,
          XFA_Attribute::Thickness,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Corner::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe8dddf50,
          L"join",
          &CJX_Corner::join,
          XFA_Attribute::Join,
          XFA_ScriptType::Basic)},
    {ATTR(0xe948b9a8,
          L"radius",
          &CJX_Corner::radius,
          XFA_Attribute::Radius,
          XFA_ScriptType::Basic)},

    /* msgId */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* color */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Color::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xabfa6c4f,
          L"cSpace",
          &CJX_Color::cSpace,
          XFA_Attribute::CSpace,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Color::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Color::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* keep */
    {ATTR(0x3848b3f,
          L"next",
          &CJX_Keep::next,
          XFA_Attribute::Next,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Keep::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x6a3405dd,
          L"previous",
          &CJX_Keep::previous,
          XFA_Attribute::Previous,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Keep::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b59543,
          L"intact",
          &CJX_Keep::intact,
          XFA_Attribute::Intact,
          XFA_ScriptType::Basic)},

    /* query */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Query::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x268b7ec1,
          L"commandType",
          &CJX_Query::commandType,
          XFA_Attribute::CommandType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Query::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* insert */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Insert::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Insert::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* imageEdit */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ImageEdit::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ImageEdit::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbde9abda,
          L"data",
          &CJX_ImageEdit::data,
          XFA_Attribute::Data,
          XFA_ScriptType::Basic)},

    /* validate */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Validate::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5b707a35,
          L"scriptTest",
          &CJX_Validate::scriptTest,
          XFA_Attribute::ScriptTest,
          XFA_ScriptType::Basic)},
    {ATTR(0x6b6ddcfb,
          L"nullTest",
          &CJX_Validate::nullTest,
          XFA_Attribute::NullTest,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Validate::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xe64b1129,
          L"formatTest",
          &CJX_Validate::formatTest,
          XFA_Attribute::FormatTest,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* digestMethods */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_DigestMethods::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_DigestMethods::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_DigestMethods::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* numberPatterns */

    /* pageSet */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_PageSet::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8c99377e,
          L"relation",
          &CJX_PageSet::relation,
          XFA_Attribute::Relation,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_PageSet::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_PageSet::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* integer */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Integer::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Integer::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Integer::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Integer::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* soapAddress */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_SoapAddress::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_SoapAddress::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* equate */
    {ATTR(0x25363,
          L"to",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::To,
          XFA_ScriptType::Basic)},
    {ATTR(0x66642f8f,
          L"force",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Force,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xcd7f7b54,
          L"from",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::From,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* formFieldFilling */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pageRange */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* update */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Update::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Update::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* connectString */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ConnectString::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ConnectString::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* mode */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* layout */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* #xml */
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* xsdConnection */
    {ATTR(0x2b5df51e,
          L"dataDescription",
          &CJX_XsdConnection::dataDescription,
          XFA_Attribute::DataDescription,
          XFA_ScriptType::Basic)},

    /* traverse */
    {ATTR(0xbb8df5d,
          L"ref",
          &CJX_Traverse::ref,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Traverse::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x226ca8f1,
          L"operation",
          &CJX_Traverse::operation,
          XFA_Attribute::Operation,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Traverse::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* encodings */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Encodings::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Encodings::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Encodings::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* template */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* acrobat */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* validationMessaging */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* signing */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Signing::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Signing::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Signing::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* dataWindow */
    {ATTR(0xfb67185,
          L"recordsBefore",
          &CJX_DataWindow::recordsBefore,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x21d5dfcb,
          L"currentRecordNumber",
          &CJX_DataWindow::currentRecordNumber,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x312af044,
          L"recordsAfter",
          &CJX_DataWindow::recordsAfter,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x6aab37cb,
          L"isDefined",
          &CJX_DataWindow::isDefined,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* script */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Script::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          L"contentType",
          &CJX_Script::contentType,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x6cfa828a,
          L"runAt",
          &CJX_Script::runAt,
          XFA_Attribute::RunAt,
          XFA_ScriptType::Basic)},
    {ATTR(0xa021b738,
          L"stateless",
          &CJX_Script::stateless,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Script::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xadc4c77b,
          L"binding",
          &CJX_Script::binding,
          XFA_Attribute::Binding,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Script::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Script::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* addViewerPreferences */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* alwaysEmbed */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* passwordEdit */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_PasswordEdit::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a0cc471,
          L"passwordChar",
          &CJX_PasswordEdit::passwordChar,
          XFA_Attribute::PasswordChar,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_PasswordEdit::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          L"hScrollPolicy",
          &CJX_PasswordEdit::hScrollPolicy,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* numericEdit */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_NumericEdit::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_NumericEdit::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          L"hScrollPolicy",
          &CJX_NumericEdit::hScrollPolicy,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* encryptionMethod */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* change */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pageArea */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_PageArea::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x14a32d52,
          L"pagePosition",
          &CJX_PageArea::pagePosition,
          XFA_Attribute::PagePosition,
          XFA_ScriptType::Basic)},
    {ATTR(0x8340ea66,
          L"oddOrEven",
          &CJX_PageArea::oddOrEven,
          XFA_Attribute::OddOrEven,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_PageArea::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xa85e74f3,
          L"initialNumber",
          &CJX_PageArea::initialNumber,
          XFA_Attribute::InitialNumber,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_PageArea::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe9ba472,
          L"numbered",
          &CJX_PageArea::numbered,
          XFA_Attribute::Numbered,
          XFA_ScriptType::Basic)},
    {ATTR(0xd70798c2,
          L"blankOrNotBlank",
          &CJX_PageArea::blankOrNotBlank,
          XFA_Attribute::BlankOrNotBlank,
          XFA_ScriptType::Basic)},

    /* submitUrl */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Object::Script_Som_DefaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* oids */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Oids::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Oids::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Oids::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* signature */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Signature::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Signature::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* aDBE_JSConsole */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* caption */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Caption::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x34ae103c,
          L"reserve",
          &CJX_Caption::reserve,
          XFA_Attribute::Reserve,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Caption::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Caption::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf2009339,
          L"placement",
          &CJX_Caption::placement,
          XFA_Attribute::Placement,
          XFA_ScriptType::Basic)},

    /* relevant */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* flipLabel */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* exData */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ExData::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          L"contentType",
          &CJX_ExData::contentType,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x54fa722c,
          L"transferEncoding",
          &CJX_ExData::transferEncoding,
          XFA_Attribute::TransferEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_ExData::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ExData::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc4547a08,
          L"maxLength",
          &CJX_ExData::maxLength,
          XFA_Attribute::MaxLength,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Object::Script_Som_DefaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xdb55fec5,
          L"href",
          &CJX_ExData::href,
          XFA_Attribute::Href,
          XFA_ScriptType::Basic)},

    /* dayNames */
    {ATTR(0x29418bb7,
          L"abbr",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Abbr,
          XFA_ScriptType::Basic)},

    /* soapAction */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_SoapAction::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_SoapAction::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* defaultTypeface */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf531b059,
          L"writingScript",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::WritingScript,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* manifest */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Manifest::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1b8dce3e,
          L"action",
          &CJX_Manifest::action,
          XFA_Attribute::Action,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Manifest::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Manifest::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* overflow */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Overflow::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          L"trailer",
          &CJX_Overflow::trailer,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Overflow::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_Overflow::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          L"leader",
          &CJX_Overflow::leader,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* linear */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Linear::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Linear::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Linear::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* currencySymbol */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* delete */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Delete::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Delete::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* deltas */

    /* digestMethod */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_DigestMethod::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_DigestMethod::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* instanceManager */
    {ATTR(0xb3543a6,
          L"max",
          &CJX_InstanceManager::max,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xb356ca4,
          L"min",
          &CJX_InstanceManager::min,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x6f544d49,
          L"count",
          &CJX_InstanceManager::count,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* equateRange */
    {ATTR(0x25363,
          L"to",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::To,
          XFA_ScriptType::Basic)},
    {ATTR(0xa0933954,
          L"unicodeRange",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::UnicodeRange,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xcd7f7b54,
          L"from",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::From,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* medium */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Medium::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x4ef3d02c,
          L"orientation",
          &CJX_Medium::orientation,
          XFA_Attribute::Orientation,
          XFA_ScriptType::Basic)},
    {ATTR(0x65e30c67,
          L"imagingBBox",
          &CJX_Medium::imagingBBox,
          XFA_Attribute::ImagingBBox,
          XFA_ScriptType::Basic)},
    {ATTR(0x9041d4b0,
          L"short",
          &CJX_Medium::shortValue,
          XFA_Attribute::Short,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Medium::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe349d044,
          L"stock",
          &CJX_Medium::stock,
          XFA_Attribute::Stock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b4afb0,
          L"long",
          &CJX_Medium::longValue,
          XFA_Attribute::Long,
          XFA_ScriptType::Basic)},

    /* textEdit */
    {ATTR(0x5ce6195,
          L"vScrollPolicy",
          &CJX_TextEdit::vScrollPolicy,
          XFA_Attribute::VScrollPolicy,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_TextEdit::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1ef3a64a,
          L"allowRichText",
          &CJX_TextEdit::allowRichText,
          XFA_Attribute::AllowRichText,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a32e493,
          L"multiLine",
          &CJX_TextEdit::multiLine,
          XFA_Attribute::MultiLine,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_TextEdit::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          L"hScrollPolicy",
          &CJX_TextEdit::hScrollPolicy,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* templateCache */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd52482e0,
          L"maxEntries",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::MaxEntries,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* compressObjectStream */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* dataValue */
    {ATTR(0x42fed1fd,
          L"contentType",
          &CJX_DataValue::contentType,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x8855805f,
          L"contains",
          &CJX_DataValue::contains,
          XFA_Attribute::Contains,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_DataValue::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_DataValue::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xe372ae97,
          L"isNull",
          &CJX_DataValue::isNull,
          XFA_Attribute::IsNull,
          XFA_ScriptType::Basic)},

    /* accessibleContent */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* nodeList */

    /* includeXDPContent */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xmlConnection */
    {ATTR(0x2b5df51e,
          L"dataDescription",
          &CJX_XmlConnection::dataDescription,
          XFA_Attribute::DataDescription,
          XFA_ScriptType::Basic)},

    /* validateApprovalSignatures */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* signData */
    {ATTR(0xbb8df5d,
          L"ref",
          &CJX_SignData::ref,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_SignData::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x226ca8f1,
          L"operation",
          &CJX_SignData::operation,
          XFA_Attribute::Operation,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_SignData::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_SignData::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},

    /* packets */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* datePattern */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* duplexOption */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* base */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* bind */
    {ATTR(0xbb8df5d,
          L"ref",
          &CJX_Bind::ref,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Bind::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          L"contentType",
          &CJX_Bind::contentType,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x54fa722c,
          L"transferEncoding",
          &CJX_Bind::transferEncoding,
          XFA_Attribute::TransferEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Bind::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf197844d,
          L"match",
          &CJX_Bind::match,
          XFA_Attribute::Match,
          XFA_ScriptType::Basic)},

    /* compression */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* user */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_User::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_User::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* rectangle */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Rectangle::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Rectangle::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          L"hand",
          &CJX_Rectangle::hand,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* effectiveOutputPolicy */
    {ATTR(0x21aed,
          L"id",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* aDBE_JSDebugger */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* acrobat7 */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* interactive */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* locale */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* currentPage */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* data */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* date */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Date::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Date::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Date::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Date::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* desc */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Desc::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Desc::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* encrypt */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Encrypt::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x28dee6e9,
          L"format",
          &CJX_Encrypt::format,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Encrypt::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* draw */
    {ATTR(0x68, L"h", &CJX_Draw::h, XFA_Attribute::H, XFA_ScriptType::Basic)},
    {ATTR(0x77, L"w", &CJX_Draw::w, XFA_Attribute::W, XFA_ScriptType::Basic)},
    {ATTR(0x78, L"x", &CJX_Draw::x, XFA_Attribute::X, XFA_ScriptType::Basic)},
    {ATTR(0x79, L"y", &CJX_Draw::y, XFA_Attribute::Y, XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          L"hAlign",
          &CJX_Draw::hAlign,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Draw::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2ee7678f,
          L"rotate",
          &CJX_Draw::rotate,
          XFA_Attribute::Rotate,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Draw::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          L"vAlign",
          &CJX_Draw::vAlign,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          L"maxH",
          &CJX_Draw::maxH,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          L"maxW",
          &CJX_Draw::maxW,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          L"minH",
          &CJX_Draw::minH,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          L"minW",
          &CJX_Draw::minW,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_Draw::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xa03cf627,
          L"rawValue",
          &CJX_Draw::rawValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Draw::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          L"colSpan",
          &CJX_Draw::colSpan,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Draw::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc8fa350,
          L"locale",
          &CJX_Draw::locale,
          XFA_Attribute::Locale,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          L"anchorType",
          &CJX_Draw::anchorType,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},

    /* encryption */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* meridiemNames */

    /* messaging */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* speak */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Speak::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x39cdb0a2,
          L"priority",
          &CJX_Speak::priority,
          XFA_Attribute::Priority,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Speak::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xeb511b54,
          L"disable",
          &CJX_Speak::disable,
          XFA_Attribute::Disable,
          XFA_ScriptType::Basic)},

    /* dataGroup */

    /* common */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* #text */
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* paginationOverride */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* reasons */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Reasons::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Reasons::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Reasons::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* signatureProperties */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_SignatureProperties::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_SignatureProperties::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* threshold */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* appearanceFilter */
    {ATTR(0x21aed,
          L"id",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* fill */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Fill::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Fill::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Fill::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* font */
    {ATTR(0xcb0ac9,
          L"lineThrough",
          &CJX_Font::lineThrough,
          XFA_Attribute::LineThrough,
          XFA_ScriptType::Basic)},
    {ATTR(0x2c1c7f1,
          L"typeface",
          &CJX_Font::typeface,
          XFA_Attribute::Typeface,
          XFA_ScriptType::Basic)},
    {ATTR(0x8c74ae9,
          L"fontHorizontalScale",
          &CJX_Font::fontHorizontalScale,
          XFA_Attribute::FontHorizontalScale,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Font::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2cd79033,
          L"kerningMode",
          &CJX_Font::kerningMode,
          XFA_Attribute::KerningMode,
          XFA_ScriptType::Basic)},
    {ATTR(0x3a0273a6,
          L"underline",
          &CJX_Font::underline,
          XFA_Attribute::Underline,
          XFA_ScriptType::Basic)},
    {ATTR(0x4873c601,
          L"baselineShift",
          &CJX_Font::baselineShift,
          XFA_Attribute::BaselineShift,
          XFA_ScriptType::Basic)},
    {ATTR(0x4b319767,
          L"overlinePeriod",
          &CJX_Font::overlinePeriod,
          XFA_Attribute::OverlinePeriod,
          XFA_ScriptType::Basic)},
    {ATTR(0x79543055,
          L"letterSpacing",
          &CJX_Font::letterSpacing,
          XFA_Attribute::LetterSpacing,
          XFA_ScriptType::Basic)},
    {ATTR(0x8ec6204c,
          L"lineThroughPeriod",
          &CJX_Font::lineThroughPeriod,
          XFA_Attribute::LineThroughPeriod,
          XFA_ScriptType::Basic)},
    {ATTR(0x907c7719,
          L"fontVerticalScale",
          &CJX_Font::fontVerticalScale,
          XFA_Attribute::FontVerticalScale,
          XFA_ScriptType::Basic)},
    {ATTR(0xa686975b,
          L"size",
          &CJX_Font::size,
          XFA_Attribute::Size,
          XFA_ScriptType::Basic)},
    {ATTR(0xb5e49bf2,
          L"posture",
          &CJX_Font::posture,
          XFA_Attribute::Posture,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Font::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbd6e1d88,
          L"weight",
          &CJX_Font::weight,
          XFA_Attribute::Weight,
          XFA_ScriptType::Basic)},
    {ATTR(0xbd96a0e9,
          L"underlinePeriod",
          &CJX_Font::underlinePeriod,
          XFA_Attribute::UnderlinePeriod,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0ec9fa4,
          L"overline",
          &CJX_Font::overline,
          XFA_Attribute::Overline,
          XFA_ScriptType::Basic)},

    /* form */
    {ATTR(0xaf754613,
          L"checksum",
          &CJX_Object::Script_Form_Checksum,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* mediumInfo */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* certificate */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Certificate::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Certificate::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* password */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Password::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Password::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* runScripts */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* trace */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* float */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Float::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Float::defaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Float::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Float::value,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* renderPolicy */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* logPseudoModel */

    /* destination */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* value */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Value::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          L"relevant",
          &CJX_Value::relevant,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Value::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xea7090a0,
          L"override",
          &CJX_Value::override,
          XFA_Attribute::Override,
          XFA_ScriptType::Basic)},

    /* bookend */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Bookend::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          L"trailer",
          &CJX_Bookend::trailer,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Bookend::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          L"leader",
          &CJX_Bookend::leader,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* exObject */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_ExObject::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x60a61edd,
          L"codeType",
          &CJX_ExObject::codeType,
          XFA_Attribute::CodeType,
          XFA_ScriptType::Basic)},
    {ATTR(0xb373a862,
          L"archive",
          &CJX_ExObject::archive,
          XFA_Attribute::Archive,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_ExObject::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe1a26b56,
          L"codeBase",
          &CJX_ExObject::codeBase,
          XFA_Attribute::CodeBase,
          XFA_ScriptType::Basic)},
    {ATTR(0xeb091003,
          L"classId",
          &CJX_ExObject::classId,
          XFA_Attribute::ClassId,
          XFA_ScriptType::Basic)},

    /* openAction */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* neverEmbed */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* bindItems */
    {ATTR(0x47d03490,
          L"connection",
          &CJX_BindItems::connection,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0xc39a88bd,
          L"labelRef",
          &CJX_BindItems::labelRef,
          XFA_Attribute::LabelRef,
          XFA_ScriptType::Basic)},
    {ATTR(0xd50f903a,
          L"valueRef",
          &CJX_BindItems::valueRef,
          XFA_Attribute::ValueRef,
          XFA_ScriptType::Basic)},

    /* calculate */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Calculate::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Calculate::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xea7090a0,
          L"override",
          &CJX_Calculate::override,
          XFA_Attribute::Override,
          XFA_ScriptType::Basic)},

    /* print */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* extras */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Extras::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Extras::type,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Extras::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* proto */

    /* dSigData */

    /* creator */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* connect */
    {ATTR(0xbb8df5d,
          L"ref",
          &CJX_Connect::ref,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Connect::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x24d85167,
          L"timeout",
          &CJX_Connect::timeout,
          XFA_Attribute::Timeout,
          XFA_ScriptType::Basic)},
    {ATTR(0x47d03490,
          L"connection",
          &CJX_Connect::connection,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0x552d9ad5,
          L"usage",
          &CJX_Connect::usage,
          XFA_Attribute::Usage,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Connect::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc860f30a,
          L"delayedOpen",
          &CJX_Connect::delayedOpen,
          XFA_Attribute::DelayedOpen,
          XFA_ScriptType::Basic)},

    /* permissions */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* connectionSet */

    /* submit */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Submit::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x28dee6e9,
          L"format",
          &CJX_Submit::format,
          XFA_Attribute::Format,
          XFA_ScriptType::Basic)},
    {ATTR(0x824f21b7,
          L"embedPDF",
          &CJX_Submit::embedPDF,
          XFA_Attribute::EmbedPDF,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Submit::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_Submit::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xdc75676c,
          L"textEncoding",
          &CJX_Submit::textEncoding,
          XFA_Attribute::TextEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xf889e747,
          L"xdpContent",
          &CJX_Submit::xdpContent,
          XFA_Attribute::XdpContent,
          XFA_ScriptType::Basic)},

    /* range */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* linearized */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* packet */
    {ATTR(0x97be91b,
          L"content",
          &CJX_Packet::content,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* rootElement */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_RootElement::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_RootElement::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* plaintextMetadata */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* numberSymbols */

    /* printHighQuality */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* driver */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* incrementalLoad */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* subjectDN */
    {ATTR(0x4156ee3f,
          L"delimiter",
          &CJX_SubjectDN::delimiter,
          XFA_Attribute::Delimiter,
          XFA_ScriptType::Basic)},

    /* compressLogicalStructure */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* incrementalMerge */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* radial */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Radial::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Radial::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Radial::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* variables */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Variables::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Variables::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* timePatterns */

    /* effectiveInputPolicy */
    {ATTR(0x21aed,
          L"id",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* nameAttr */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* conformance */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* transform */
    {ATTR(0xbb8df5d,
          L"ref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* lockDocument */
    {ATTR(0x21aed,
          L"id",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          L"type",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* breakAfter */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_BreakAfter::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x453eaf38,
          L"startNew",
          &CJX_BreakAfter::startNew,
          XFA_Attribute::StartNew,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          L"trailer",
          &CJX_BreakAfter::trailer,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xa6118c89,
          L"targetType",
          &CJX_BreakAfter::targetType,
          XFA_Attribute::TargetType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_BreakAfter::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          L"target",
          &CJX_BreakAfter::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          L"leader",
          &CJX_BreakAfter::leader,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* line */
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Line::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xabef37e3,
          L"slope",
          &CJX_Line::slope,
          XFA_Attribute::Slope,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Line::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          L"hand",
          &CJX_Line::hand,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* list */
    {ATTR(0xa60dd202,
          L"length",
          &CJX_List::length,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* source */
    {ATTR(0x20146,
          L"db",
          &CJX_Source::db,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Source::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Source::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* occur */
    {ATTR(0xb3543a6,
          L"max",
          &CJX_Occur::max,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xb356ca4,
          L"min",
          &CJX_Occur::min,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Occur::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d0b5fca,
          L"initial",
          &CJX_Occur::initial,
          XFA_Attribute::Initial,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Occur::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* pickTrayByPDFSize */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* monthNames */
    {ATTR(0x29418bb7,
          L"abbr",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Abbr,
          XFA_ScriptType::Basic)},

    /* severity */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* groupParent */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* documentAssembly */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* numberSymbol */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* tagged */
    {ATTR(0xbe52dfbf,
          L"desc",
          &CJX_Object::Script_Attribute_String,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          L"lock",
          &CJX_Object::Script_Attribute_BOOL,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* items */
    {ATTR(0xbb8df5d,
          L"ref",
          &CJX_Items::ref,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          L"use",
          &CJX_Items::use,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          L"presence",
          &CJX_Items::presence,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0xa5b410cf,
          L"save",
          &CJX_Items::save,
          XFA_Attribute::Save,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          L"usehref",
          &CJX_Items::usehref,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* object */
    {ATTR(0xb2c80857,
          L"className",
          &CJX_Object::className,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* list */
    {ATTR(0xa60dd202,
          L"length",
          &CJX_List::length,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* [unknown] */

    /* tree */
    {ATTR(0x31b19c1,
          L"name",
          &CJX_Tree::name,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0x9f9d0f9,
          L"all",
          &CJX_Tree::all,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0x4df15659,
          L"nodes",
          &CJX_Tree::nodes,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0x78a8d6cf,
          L"classAll",
          &CJX_Tree::classAll,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0xcad6d8ca,
          L"parent",
          &CJX_Tree::parent,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0xd5679c78,
          L"index",
          &CJX_Tree::index,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xdb5b4bce,
          L"classIndex",
          &CJX_Tree::classIndex,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xe4989adf,
          L"somExpression",
          &CJX_Tree::somExpression,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},

    /* node */
    {ATTR(0x21aed,
          L"id",
          &CJX_Node::id,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x234a1,
          L"ns",
          &CJX_Node::ns,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0x50d1a9d1,
          L"model",
          &CJX_Node::model,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0xacb4823f,
          L"isContainer",
          &CJX_Node::isContainer,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xe372ae97,
          L"isNull",
          &CJX_Node::isNull,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xfe612a5b,
          L"oneOfChild",
          &CJX_Node::oneOfChild,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},

    /* [unknown] */

    /* [unknown] */

    /* model */
    {ATTR(0x97c1c65,
          L"context",
          &CJX_Model::context,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},
    {ATTR(0x58be2870,
          L"aliasNode",
          &CJX_Model::aliasNode,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Object)},

    /* [unknown] */
    {ATTR(0xa52682bd,
          L"{ATTR(default}",
          &CJX_Object::Script_Som_DefaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          L"value",
          &CJX_Object::Script_Som_DefaultValue,
          XFA_Attribute::Unknown,
          XFA_ScriptType::Basic)},
};

#undef ATTR

const size_t g_szSomAttributeCount = FX_ArraySize(g_SomAttributeData);
